/*******************************************************************************
 * Copyright (c) 2014, 2014 Bruno Medeiros and other Contributors.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bruno Medeiros - initial API and implementation
 *******************************************************************************/
package melnorme.utilbox.misc;

import static melnorme.utilbox.core.Assert.AssertNamespace.assertFail;

import java.nio.file.InvalidPathException;
import java.nio.file.Path;
import java.nio.file.Paths;

import melnorme.utilbox.core.CommonException;


public class PathUtil {
	
	/** @return a valid path, 
	 * or null if a valid path could not be created from given pathString. */
	public static Path createPathOrNull(String pathString) {
		try {
			return Paths.get(pathString);
		} catch (InvalidPathException ipe) {
			return null;
		}
	}

	/** @return a valid path. Given pathString must represent a valid path. */
	public static Path createValidPath(String pathString) {
		try {
			return Paths.get(pathString);
		} catch (InvalidPathException ipe) {
			throw assertFail();
		}
	}

	/** @return a valid path, 
	 * or throws a checked exception if a valid path could not be created from given pathString. */
	public static Path createPath(String pathString) throws InvalidPathExceptionX {
		try {
			return Paths.get(pathString);
		} catch (InvalidPathException ipe) {
			throw new InvalidPathExceptionX(ipe);
		}
	}

	/** Checked analogue/wrapper for {@link InvalidPathException} */
	public static class InvalidPathExceptionX extends Exception {
		
		private static final long serialVersionUID = 1L;
		
		public InvalidPathExceptionX(InvalidPathException ipe) {
			super(ipe);
		}
		
	}
	
	public static Path getParentOrEmpty(Path path) throws CommonException {
		Path parent = path.getParent();
		return parent == null ? createValidPath("") : parent;
	}
	
}